#!/usr/bin/env python
#-*- coding: utf-8 -*-

"""
Mauvaise Blague. "Joël. Maïzi" <joel.maizi@lirmm.fr>

Script de récupération des données à partir de deux fichiers :
- ls : le résultat de la commande 'ls -l' fait dans un répertoire,
  le fichier doit être nettoyé des lignes qui ne représentent pas
  les fichiers du répertoire.
- bundle : la concaténation des fichiers listés dans le fichier ls.
  Attention ! L'ordre doit être le même que dans le fichier ls.
"""

import sys
import argparse

class Ls:
    def __init__(self, ls_line):
        ls_list = ls_line.split()
        self.rights = ls_list[0]
        self.nlinks = ls_list[1]
        self.owner = ls_list[2]
        self.group = ls_list[3]
        self.size = int(ls_list[4])
        self.month = ls_list[5]
        self.day = ls_list[6]
        self.hour = ls_list[7]
        self.name = ls_list[8]

class Bundle:
    def __init__(self, directory):
        self.__dir = directory
        try:
            with open('{}/ls'.format(self.__dir)) as f:
                self.__lss = [Ls(ls_line) for ls_line in f]
        except OSError as err:
            print(err)
            sys.exit()
        self.__filenames = [ls.name for ls in self.__lss]

    def list(self):
        for ls in self.__lss:
            print("{}: {}".format(ls.name, ls.size))

    def __index(self, filename):
        """Retourne l'indice du fichier filename dans bundle (débute à 0)
        """
        return self.__filenames.index(filename)

    def __offset(self, filename):
        """Retourne la position du premier octet du fichier filename
        """
        return sum([self.__lss[i].size for i in range(self.__index(filename))])

    def __size(self, filename):
        """Retourne la taille du fichier filename
        """
        return self.__lss[self.__index(filename)].size

    def has_file(self, filename):
        """Retourne True si filename fait bien partie des fichiers contenus
        """
        return filename in self.__filenames

    def __content(self, filename):
        """Le contenu retourné est de type "bytes"
        """
        with open('{}/bundle'.format(self.__dir), "rb") as f:
            offset = self.__offset(filename)
            size = self.__size(filename)
            f.seek(offset)
            return f.read(size)

    def write(self, filename):
        """Enregistre le fichier sur le disque (dans le répertoire d'origine)
        """
        assert(filename in self.__filenames)
        open("{}/{}".format(self.__dir, filename), "wb").write(
            self.__content(filename))

    def cat(self, filename):
        """Le contenu (bytes) est décodé avant d'être affiché
        """
        assert(filename in self.__filenames)
        print(self.__content(filename).decode())

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "repertoire", help="Répertoire contenant les fichiers ls et bundle")
    parser.add_argument(
        "fichiers",
        help="Nom du(des) fichier(s) à afficher ou enregistrer", nargs="*")
    parser.add_argument(
        "-w", "--write", help="Restaure le(s) fichier(s)", action="store_true")
    args = parser.parse_args()
    directory = args.repertoire
    fichiers = args.fichiers
    write = args.write
    bundle = Bundle(directory)
    if not fichiers:
        bundle.list()
    else:
        for f_name in fichiers:
            if not bundle.has_file(f_name):
                print("{} n'est pas dans bundle!".format(f_name))
                continue
            if not args.write:
                bundle.cat(f_name)
            else:
                bundle.write(f_name)
